"""remove snkmt_db_version

Revision ID: addc0edea736
Revises: a088a7b93fe5
Create Date: 2025-09-13 20:32:10.389505

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "addc0edea736"
down_revision: Union[str, None] = "a088a7b93fe5"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("snkmt_db_version")
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "snkmt_db_version",
        sa.Column("id", sa.VARCHAR(), nullable=False),
        sa.Column("major", sa.INTEGER(), nullable=False),
        sa.Column("minor", sa.INTEGER(), nullable=False),
        sa.Column("timestamp", sa.DATETIME(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###
